"
 Copyright (c) 2002 David Forster <dforste89@yahoo.com>

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation files
 (the ``Software''), to deal in the Software without restriction,
 including without limitation the rights to use, copy, modify, merge,
 publish, distribute, sublicense, and/or sell copies of the Software,
 and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED ``AS IS'', WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.

"

CStruct subclass: #Display
	    declaration: #( )
	    classVariableNames: ''
	    poolDictionaries: ''
	    category: 'Xlib-Display'!


Display comment:'A Display object represents the connection wit the X
Server.  It''s primary task is to generate and process X Protocol
packets.'!

!Display class methodsFor: 'instance creation'!
new
    "Return a Display object for the default X Server."
    ^self openDisplay: nil
!

openDisplay: aString
    "Open a new connection with the given X server.  If `nil' is given
    then the environment variable `DISPLAY' will be used."
    ^self cXOpenDisplay: aString.
!!

!Display methodsFor: 'initialization and finalization'!
finalize
    "We are not a normal struct, so don't call CStruct's normal
    finalize method.  Close the display. "
    self close.
    ^self.
!!

!Display methodsFor: 'connection management'!

close
    "I don't know how to implement this properly.  Ideally, all
    references to this object would become `nil'..."
    self cXCloseDisplay: self.
    ^nil.
!

flush
    "Flush to output buffer."
    self cXFlush: self.
    ^self
!

sync
    "Flush the output buffer and block until all requests have been
    processed."
    self cXSync: self a2: 0.
    ^self
!

discardAndSync
    "Flush the output buffer and block until all requests have been
    process.  Discard all incoming events, including those present
    before discardAndSync was called."
    self cXSync: self a2: 1.
    ^self.
!

connectionNumber
    "Return the connection number.  On POSIX systems, this is the file
    descriptor."
    ^self cXConnectionNumber: self
!

maxRequestSize
    "Return the maximum request size in 32-bit units.  Guaranteed to
    be no smaller then 16384 bytes."
    ^self cXMaxRequestSize: self	
!

extendedMaxRequestSize
    "Reutnr the maximum request size for extended data packets, in
    32-bit units.  If the X Server does not support the extended data
    protocol, then 0 is returned."
    ^self cXExtendedMaxRequestSize: self.
!

lastKnownRequestProcessed
    "Return the serial number of the last request known the been
    processed by the X Server."
    ^self cXLastKnownRequestProcessed: self.
!

nextRequest
    "Return the serial number that will be used for the next request."
    ^self cXNextRequest: self
!

protocolVersion
    "Return the major version number of the X protocol associated with
    the current display."
    ^self cXProtocolVersion: self.
!

protocolRevision
    "Return the minor version number of the X protocol associated with
    the current display."
    ^self cXProtocolRevision: self.
!

serverVendor
    "Return the server vendor string for the current display."
    ^self cXServerVendor: self
!

vendorRelease
    "Return the vendor release string for the current display."
    ^self cXVendorRelease: self
!

displayString
    "Return the string used to connect to the X Server."
    ^self cXDisplayString: self
!

queueLength
    "Return the length of the event queue."
    ^self cXQLength: self.

!!

!Display methodsFor: 'event management'!
eventsQueued
    "Return the number of events in the event queue."
    ^self cXEventsQueued: self a2: 0
!

eventsQueuedAfterFlush
    "Return the number of events in the event queue.  If empty,
    perform a flush and return the count of any new events."
    ^self cXEventsQueued: self a2: 1
!

eventsQueuedAfterReading
    "Return the number of events in the event queue.  If empty,
    try to read additional events from the connection without
    performing a flush."
    ^self cXEventsQueued: self a2: 1
!

pending
    "Return the number of events in the event queue.  If empty,
    perform a flush and return the count of any new events."
    ^self cXPending: self.
!

placeNextEventIn: anAnyEventStruct
    "Remove the next event of the event queue and place it in
    anAnyEventStruct."
    self cXNextEvent: self a2: anAnyEventStruct.
    ^anAnyEventStruct
!

nextEvent
    "Return and remove the next evnet on the event queue."

    | event |

    event := AnyEvent new.
    ^self placeNextEventIn: event	
!

peekEvent
    "Return the next event in the event queue with out removing it."
    
    | event |
    event := AnyEvent new.
    self cXPeekEvent: self a2: event.
    ^self
!

placeNextEventFor: aWindow withEventMask: eventMask in: anAnyEvent
    "Return the next event for aWindow and remove it from the queue,
    without modifying any other events.  If none are present, flush
    the connection and block until one arrives."

    self cXWindowEvent: self a2: aWindow a3: eventMask
	 a4: anAnyEvent.
    ^anAnyEvent
!!

!Display methodsFor: 'window creation'!
createWindowWithParent: aWindow withDimentions: aRectangle
    withBorder: border withDepth: depth withClass: cls
    withAttirbutes: anWinAttribute
    "Create a new window with the given properties."
    ^self notYetImplemented.
!

createSimpleWindowWithParent: parent dimensions: dimensions
    borderWidth: borderWidth border: border background: background

    "Create a simple window.  Useful for learning or quick output."

    ^self cXCreateSimpleWindow: self a2: parent a3: (dimensions left)
	  a4: (dimensions top) a5: (dimensions width)
	  a6: (dimensions height) a7: borderWidth a8: border
	  a9: background.
!!

!Display methodsFor: 'window management'!

mapWindow: window
    "Map the given window."
    self cXMapWindow: self a2: window.
    ^self
!

unmapWindow: window
    "Unmap the given window."
    self cXUnmapWindow: self a2: window.
    ^self
!!

!Display methodsFor: 'screen information'!
blackPixelOfScreen: aScreen
    "Return the black pixel value for the given screen."
    ^self cXBlackPixel: self a2: aScreen.
!
    
whitePixelOfScreen: aScreen
    "Return the white pixel value for the given screen."
    ^self cXWhitePixel: self a2: aScreen.
!

defaultColormapOfScreen: aScreen
    "Return the default colormap for the given screen."
    ^self cXDefaultColormap: self a2: aScreen.
!

defaultVisualOfScreen: aScreen
    "Return the default visual of the given screen."
    ^self cXDefaultVisual: self a2: aScreen
!

defualtDepthOfScreen: aScreen
    "Return the default depth of the given screen."
    ^self cXDefaultDepth: self a2: aScreen
!

listDepthsOfScreen: aScreen
    "Return a list of all depths supported by the given screen."
    ^self notYetImplemented.
!

defaultGCOfScreen: aScreen
    "Return the default GC of the given screen."
    ^self cXDefaultGC: self a2: aScreen.
!

defaultRootWindow
    "Return the default root window."
    ^self cXDefaultRootWindow: self.
!

defaultScreen
    "Return the default screen."
    ^self cXDefaultScreen: self
!

displayCellsOfScreen: aScreen
    "Return the number of display cells for the given screen."
    ^self cXDefaultDisplayCells: self a2: aScreen
!

rootWindowDepthOfScreen: aScreen
    "Return the depth of the root window for the given screen."
    ^self cXDisplayPlanes: self a2: aScreen
!

rootWindowOfScreen: aScreen
    "Return the root window of the given screen."
    ^self cXRootWindow: self a2: aScreen
!

screenCount
    "Return the number of screens supported by the display."
    ^self cXScreenCount: self
!!

!Display methodsFor: 'rendering'!

drawRectangle: aRectangle on: drawable withGC: gc 
    "Draw a rectangle."
    self cXDrawRectangle: self a2: drawable a3: gc 
	 a4: (aRectangle left) a5: (aRectangle top)
	 a6: (aRectangle width) a7: (aRectangle right).
    ^self
	
!!
